import { KLineData } from 'klinecharts';
import { OverlayTemplate } from 'klinecharts/src/component/Overlay.js';

const enum OrderDirection {
  BUY = 'buy',
  SELL = 'sell'
}

interface OrderAttr {
  candle: KLineData
  direction: OrderDirection
  color: string
  textColor: string
}

const marketOrder: OverlayTemplate = {
  name: 'marketOrder',
  createPointFigures: ({ overlay, coordinates, yAxis }) => {
    const attr = overlay.extendData(overlay) as OrderAttr
    const text = attr.direction == OrderDirection.BUY ? 'BUY' : 'SELL';
    const triangleSize = 15;

    const startX = coordinates[0].x;
    const startY = coordinates[0].y;
    const lineEndX = startX + 20;

    let yAnchor = yAxis?.convertToPixel(attr.direction === OrderDirection.BUY ? attr.candle.low : attr.candle.high) || startY;
    if (attr.direction === OrderDirection.BUY) {
      yAnchor += 5;
    } else {
      yAnchor -= 5;
    }

    const figures = [
      {
        type: 'line',
        attrs: { coordinates: [{ x: startX, y: startY }, { x: lineEndX, y: startY }] },
        styles: { color: attr.color },
        ignoreEvent: true
      }
    ];

    if (attr.direction == OrderDirection.BUY) {
      figures.push({
        type: 'text',
        attrs: { x: startX, y: yAnchor + triangleSize + 5, text, align: 'center', baseline: 'top' },
        styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
        ignoreEvent: true
      });

      figures.push({
        type: 'polygon',
        attrs: { coordinates: [{ x: startX, y: yAnchor }, { x: startX + triangleSize / 2, y: yAnchor + triangleSize }, { x: startX - triangleSize / 2, y: yAnchor + triangleSize }] },
        styles: { color: attr.color },
        ignoreEvent: true
      });
    } else {
      figures.push({
        type: 'text',
        attrs: { x: startX, y: yAnchor - triangleSize - 2, text, align: 'center', baseline: 'bottom' },
        styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
        ignoreEvent: true
      });

      figures.push({
        type: 'polygon',
        attrs: { coordinates: [{ x: startX - triangleSize / 2, y: yAnchor - triangleSize }, { x: startX + triangleSize / 2, y: yAnchor - triangleSize }, { x: startX, y: yAnchor }] },
        styles: { color: attr.color },
        ignoreEvent: true
      });
    }

    return figures;
  }
}

export default marketOrder
