import { KLineData, LineType } from 'klinecharts';
import { OverlayTemplate } from 'klinecharts/src/component/Overlay.js';

const enum OrderDirection {
  BUY = 'buy',
  SELL = 'sell'
}

const enum OrderStatus {
  PENDING = 'pending',
  FILLED = 'filled'
}

interface MarketStopAttr {
  candle: KLineData
  direction: OrderDirection
  color: string
  textColor: string
  status: OrderStatus
}

const marketStop: OverlayTemplate = {
  name: 'marketStop',
  createPointFigures: ({ overlay, coordinates, yAxis, bounding }) => {
    const attr = overlay.extendData(overlay) as MarketStopAttr
    let text = 'SL';

    const triangleSize = 15;

    const startX = coordinates[0].x;
    const startY = coordinates[0].y;
    let lineEndX = startX + 20;
    let lineStyle = 'solid';

    if (attr.status == OrderStatus.PENDING) {
      lineEndX = bounding.width;
      lineStyle = 'dashed';
    }

    if (attr.status == OrderStatus.FILLED) {
      text = '❌';
    }

    let yAnchor = yAxis?.convertToPixel(attr.direction === OrderDirection.SELL ? attr.candle.low : attr.candle.high) || startY;
    if (attr.direction === OrderDirection.SELL) {
      yAnchor += 5;
    } else {
      yAnchor -= 5;
    }

    const figures = [
      {
        type: 'line',
        attrs: { coordinates: [{ x: startX, y: startY }, { x: lineEndX, y: startY }] },
        styles: { color: attr.color, style: lineStyle },
        ignoreEvent: true
      },
    ];

    if (attr.direction == OrderDirection.BUY) {
      if (attr.status == OrderStatus.PENDING) {
        figures.push({
          type: 'text',
          attrs: { x: bounding.width - 5, y: startY, text, align: 'center', baseline: 'bottom' },
          styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
          ignoreEvent: true
        });
      } else {
        figures.push({
          type: 'text',
          attrs: { x: startX + 5, y: startY - 5, text, align: 'left', baseline: 'bottom' },
          styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
          ignoreEvent: true
        });
      }
    } else {
      if (attr.status == OrderStatus.PENDING) {
        figures.push({
          type: 'text',
          attrs: { x: bounding.width - 5, y: startY , text, align: 'right', baseline: 'bottom' },
          styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
          ignoreEvent: true
        });
      } else {
        figures.push({
          type: 'text',
          attrs: { x: startX + 5, y: startY + 5, text, align: 'left', baseline: 'top' },
          styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
          ignoreEvent: true
        });
      }
    }

    return figures;
  }
}

export default marketStop
