import { KLineData } from 'klinecharts';
import { OverlayTemplate } from 'klinecharts/src/component/Overlay.js';

const enum Align {
  ABOVE = 'above',
  BELOW = 'below'
}

interface OrderAttr {
  candle: KLineData
  align: Align
  text: string
  color: string
  textColor: string
}

const tag: OverlayTemplate = {
  name: 'tag',
  createPointFigures: ({ overlay, coordinates, yAxis }) => {
    const attr = overlay.extendData(overlay) as OrderAttr
    const triangleSize = 15;

    const startX = coordinates[0].x;
    const startY = coordinates[0].y;

    let yAnchor = yAxis?.convertToPixel(attr.align === Align.BELOW ? attr.candle.low : attr.candle.high) || startY;
    if (attr.align === Align.BELOW) {
      yAnchor += 5;
    } else {
      yAnchor -= 5;
    }

    if (attr.align == Align.BELOW) {
      return [
        {
          type: 'text',
          attrs: { x: startX, y: yAnchor + triangleSize + 5, text: attr.text, align: 'center', baseline: 'top' },
          styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
          ignoreEvent: true
        },
        {
          type: 'polygon',
          attrs: { coordinates: [{ x: startX, y: yAnchor }, { x: startX + triangleSize / 2, y: yAnchor + triangleSize }, { x: startX - triangleSize / 2, y: yAnchor + triangleSize }] },
          styles: { color: attr.color },
          ignoreEvent: true
        }
      ];
    } else {
      return [
        {
          type: 'text',
          attrs: { x: startX, y: yAnchor - triangleSize - 2, text: attr.text, align: 'center', baseline: 'bottom' },
          styles: { color: attr.color || '#ffffff', family: 'sans serif', weight: 'bold' },
          ignoreEvent: true
        },
        {
          type: 'polygon',
          attrs: { coordinates: [{ x: startX - triangleSize / 2, y: yAnchor - triangleSize }, { x: startX + triangleSize / 2, y: yAnchor - triangleSize }, { x: startX, y: yAnchor }] },
          styles: { color: attr.color },
          ignoreEvent: true
        }
      ]
    }
  }
}

export default tag
